import { PropOptions } from "vue";
import { Prop } from "vue/types/options";
export declare type AnyFunc = (...args: any[]) => any;
export declare type Validator<T> = (value: T) => boolean;
export interface Finisher<T> {
    readonly required: PropOptions<T> & {
        required: true;
    };
    readonly optional: PropOptions<T | undefined>;
    default(value: T | (() => T & object)): PropOptions<T>;
}
export interface Builder<T> extends Finisher<T> {
    validator(validator: Validator<T>): Finisher<T>;
}
export interface ChainableBuilder<T> extends Builder<T> {
    readonly or: BuilderCollection<T>;
}
export declare type GeneralBuilder<BaseType> = <T1, T2 = never, T3 = never, T4 = never, T5 = never>(p1: Prop<T1>, p2?: Prop<T2>, p3?: Prop<T3>, p4?: Prop<T4>, p5?: Prop<T5>) => ChainableBuilder<BaseType | T1 | T2 | T3 | T4 | T5>;
export interface NamedBuilders<BaseType> {
    ofFunction<T extends AnyFunc>(): ChainableBuilder<BaseType | T>;
    ofArray<T>(): ChainableBuilder<BaseType | T[]>;
    ofRoArray<T>(): ChainableBuilder<BaseType | ReadonlyArray<T>>;
    ofObject<T extends object>(): ChainableBuilder<BaseType | T>;
    ofStringLiterals<T extends string>(...values: T[]): Finisher<T>;
    ofAny(): Builder<any>;
    ofType<T>(): Builder<T>;
}
export declare type BuilderCollection<BaseType> = GeneralBuilder<BaseType> & NamedBuilders<BaseType>;
declare const rootBuilders: BuilderCollection<never>;
export default rootBuilders;
